#!/usr/bin/perl
#
# Copyright 2000, Cobalt Networks, Inc
# All rights reservedd
#
# changes the "administrivia=yes" line in the config files for all mailing
# lists hosted on machine


use Cobalt::Vsite;
use Cobalt::List;
use File::Copy;
use Cobalt::Product;
use Cobalt::Util;

my  @vhosts=vsite_list();
foreach $vhost (@vhosts) {
	my @lists=list_lists($$vhost[2],$$vhost[1]);
	print "here";
	foreach $list (@lists) {
		list_config_update($$vhost[2], $$vhost[1], $list,
			"administrivia", "no");
	}
}

##
# A few of the functions needed to do this fix properly are privatei
# functions in Cobalt::List

sub list_config_update
# Update the config info for a list
# Arguments: virtual site group, virtual site fqdn, list name, 
#            config variable regex, config value
# Return value: 1 on success, 0 means failure
{
    my ($group, $fqdn, $list, $regex, $val) = @_;
    my ($dir) = list_virtualize ($group, $fqdn);
    my $lock;
    $list =~ tr[A-Z][a-z];	# lowercase the characters
    return $lock if (($lock = lock_aliases()) !~ /^2/);
    open (LIST, "< $dir/lists/$list.config") or
	abort(MSG_error("list_cant_read_config", $list));
    open (LIST_NEW, "> $Lockdir/$list.config") ||
	abort(MSG_error("list_cant_write_config", $Lockdir));
    while (<LIST>) {
	if (/^$regex/) {
	    print LIST_NEW "$& = $val\n";
	}
	else {
	    print LIST_NEW;
	}
    }
    close LIST;
    close LIST_NEW;
    move("$Lockdir/$list.config", "$dir/lists/$list.config") or
	abort(MSG_error("list_cant_move_config"));
    chmod 0660, "$dir/lists/$list.config";
    chown (scalar getpwnam($Majordomo_user), 
	   scalar getgrnam($group ? $group : $Majordomo_group),
	   "$dir/lists/$list.config");
    unlock_aliases();
    return (MSG_ok("list_config_update_ok"));
}

######################################################

sub lock_aliases
# Lock the aliases file so that nobody can collide with us.
# Arguments: none
# Side effects: copies /etc/aliases to /etc/altmp
{
    if (-e $Altmp)
    {
	return (MSG_error("4261"));
    }
    else
    {
	# Lock the aliases file by creating the altmp file as a
	# copy of the aliases file.
	print STDERR "Copying aliases file $Aliasfile to $Altmp\n" if ($Debug);
	copy ($Aliasfile, $Altmp);
    }
    print STDERR "Locked $Aliasfile with $Altmp\n" if ($Debug);

    open (LOCK,">$Lockdir/list-$$") or
      abort (MSG_error("4266"));
    print LOCK "$$\n$0";
    close LOCK;

    return MSG_ok("2260");
}

sub unlock_aliases
# Unlocks the aliases file by wiping altmp
# Arguments: none
# Side effects: wipes altmp
{
    unlink($Altmp) || return MSG_error("4264", $Altmp);
    print STDERR "Removed $Altmp\n" if ($Debug);
    unlink("$Lockdir/list-$$");
    return MSG_ok("2262");
}

sub abort
# leave in a graceful manner
# Arguments: a string to print
# Side effects: unlocks anything we've locked
{
    unlock_aliases()  if (-e "$Lockdir/list-$$");
    return join('',@_);
}

