#!/usr/bin/perl

# raq2 restore
# Author: Duncan Laurie [duncan@cobaltnet.com]
# (c) 1999 Cobalt Networks, Inc.
#
#
# jeff fixed a bunch of security bits. i've special-cased a few things
# to enable site admin restores without causing security problems.
# here are some details on what restore.cgi does:
#    1) 3 modes -- raq, site, or user
#       - raq and site run as root if the user is a site admin. 
#       - user runs as user.
#    2) when site mode is selected, the following things happen:
#       - unless excused, files with .. in their name are not restored
#       - the only file pattern that gets excused is 
#         /home/sites/<site>/../../spool/mail/<user> 
#       - we ignore the group parameter.
#    3) when restoring mail, both the link and the actual file get 
#       restored
#    -asun

use Cobalt::Language;
use Cobalt::Locale;
### START HARDCODED PATHS
my $Adminuser = 'admin';
my $Big_tmp = '/home/tmp';
my $Mailspool = '/var/spool/mail';
my $Tar = '/bin/tar';
my $Template_dir = '/usr/admserv/templates';
my $Userdir = '/home/sites';
my $Vaca_dir = '/var/vacation';

my $Debug = (-e "/etc/DEBUG") ? 1 : 0;
$^W = $Debug;
### END HARDCODED PATHS
use Cobalt::User;
use Cobalt::Util;
use Cobalt::Group;
use Cobalt::Quota;
use Cobalt::Vsite;
use CGI;
use POSIX;
# cannot use strict because of problems posting the backup archives
#use strict;
use Locale::gettext;

my $user = $ENV{'REMOTE_USER'};
# we remain root if we're the admin user or a site admin
if ($user and not user_authok($user, "site-adm")) {
    $( = $) = (getgrnam($site))[2];
    $< = $> = (getpwnam($user))[2];
}

my $site = (user_list_site($user))[1];
my @user_stats = getpwnam($user);
my ($filename,$archive,$fileount,$restorelist,$group);
my ($bkmode,$filecount,$filelist,$tops,$file,$top,$stage);
my (@index,@index2,%files,%header,@sFiles,%badFiles);
my $index = 0;
my $bad = $Big_tmp."/restore-files-bad.$$";
my $good = $Big_tmp."/restore-files-good.$$";
my $tar = $Tar;
my $tarfile;
my $HtmlHeader;
my $icon_folder = "/cobalt-images/icon_folder";
my $icon_generic = "/cobalt-images/icon_generic";
my $icon_mail = "/cobalt-images/icon_text";
my $icon_generic_secure = "/cobalt-images/icon_generic.sec";
my $icon_folder_secure = "/cobalt-images/icon_folder.sec";
my $homedir = 0;

my %sitePaths;
my $sitehome = 	$Userdir.'/'. $site;
my $sitemailpath = '/../../spool/mail';
my $mailpath = $sitehome . $sitemailpath; #special case bleah

my $userhome = @user_stats[7];
my %userPaths = ( "$userhome"              => 0,
 		  "$Mailspool/$user"       => 1,
		  "$mailpath/$user"        => 1,
		  "$Vaca_dir/$user.msg"    => 1);

my @doubledotok;

my $locale=Cobalt::Locale->getByHTTPHeader( $ENV{ 'HTTP_ACCEPT_LANGUAGE' } );
if( $locale ) { $ENV{ 'LANGUAGE' }=$locale->getLocale(); }

my $q = new CGI;
my $BUG = 0;

if ($BUG) {
    open ERR, "> /tmp/restore.err";
}

&parse_web();

if ($stage == 0)
{
# complete restore
    print "content-type: text/html \nCache-Control: no-cache\n\n";
    print "<body bgcolor=\"#ffffff\">";
    $HtmlHeader = 1;
    &read_backupheader();
    if ($header{"backupset"} eq "site" && $user eq "admin") {
	&error(MSG_get_ui("restore_cant_admin_vsite"));
    }
    if ($homedir == 0) {
	&write_backuparchive();
    }
    close $file;
    &print_debug();
    &gen_restore_filelist();
    &do_restore();
    &do_report();
    &do_cleanup();
    exit 0;
}
elsif ($stage == 1)
{
# selective restore, phase 1
    print "content-type: text/html \nCache-Control: no-cache\n\n";
    print "<body bgcolor=\"#ffffff\">";
    $HtmlHeader = 1;
    &read_backupheader();
    if ($header{"backupset"} eq "site" && $user eq "admin") {
        &error(MSG_get_ui("restore_cant_admin_vsite"));
    }   
    if ($homedir == 0) {
	&write_backuparchive();
    }
    close $file;
    &print_debug();
    &gen_filelist();
    
    open(TMPL,"/usr/admserv/templates/parserestore.tmpl");
    while(<TMPL>)
    {
	if (/\[FILETABLE\]/o)
	{ # dump the fileset
	    print $filelist;
	    next;
	}
	s/\[GROUP\]/$group/o;
	s/\[USER_ERROR\]//o;
	s/\[ARCHIVE\]//o;
	s/\[HOMEDIR\]/$filename/o;
	s/\[FILECOUNT\]/$filecount/o;
	s/\[OWNER\]/$header{'user'}/o;
	s/\[DATEF\]/$header{'datef'}/o;
	s/\[MODE\]/$bkmode/o;

	s/\[str.([^\]]+)\]/Locale::gettext::dgettext("ui", "$1")/geo;

	print;
    }
    close TMPL;
    close ERR;

    exit 0;
}
elsif ($stage == 2)
{
    &read_backupheader();
    if ($header{"backupset"} eq "site" && $user eq "admin") {
        &error(MSG_get_ui("restore_cant_admin_vsite"));
    }   
    close $file;
    &print_debug();
    &gen_restore_filelist();
    &do_restore();
    &do_report();
    &do_cleanup();
    
    exit 0;
}

sub print_debug
{
    my ($f,$bit);
    print ERR "file: $file\n";
    print ERR "stage: $stage\n";
    print ERR "archive: $archive\n";
    print ERR "restorelist: $restorelist\n";
    print ERR "filecount: $filecount\n";
    print ERR "user: $user\n";
    print ERR "mode: $bkmode\n";
    foreach $bit (keys %header) {
	print ERR "$bit: $header{$bit}\n";
    }
    print ERR "\n";
    foreach $f (@index) {
	print ERR "$f\n";
    }
    print ERR "\n";
    foreach $f ($filelist) {
	print ERR "$f\n";
    }
    print ERR "\n";
}

sub parse_web
{
    my (@uzers,$uzer);
# stage = 1 means selective, phase 1
# stage = 2 means selective, phase 2
# stage = 0 means complete restore

    $file = $q->param('backupfile');
    $stage = $q->param('stage');
    $stage += $q->param('selective');
    $homedir = $q->param('homedirarchive');
    $restorelist = $q->param('restorelist');
    $filecount = $q->param('filecount');
    $bkmode = $q->param('mode');

    # note: $group is ignored as it's a security hole.
#   $group = $q->param('group');
    $group = $site;

    # make sure that we don't get sent some bogus bkmode
    if ($user ne $Adminuser) {
	$bkmode = 'site' unless ($bkmode eq 'site') or
	    ($bkmode eq 'user');
    }

    if ($bkmode eq 'site') {
	$sitePaths{$sitehome} = 1;
	@uzers = vsite_get_users($group);
	if ($uzers[0] =~ /^[134]\S*\s/) {
	    # substr( $uzers[0], 5 ).'<BR>';
            @uzers = ();
	}
	foreach $uzer (@uzers) {
	    $sitePaths{"$Mailspool/$uzer"} = 1;
	    $sitePaths{"$mailpath/$uzer"} = 1;
	    # double-dots are only legal for the following path
	    push @doubledotok, "$mailpath/$uzer";
	}
    }
    
    print ERR "\n**************************\nfile: $file\nstage: $stage\nhomedirarchive: $homedir\nrestorelist: $restorelist\nfilecount: $filecount\nmode: $bkmode\n**************************\n\n" if ($BUG);
    
    unless ( $file ) {
	if ( $homedir ) {
	    print ERR "file previously uploaded...\n" if ($BUG);
	    print ERR "filename: $homedir\n" if ($BUG);
	    $filename = $homedir;
	    $homedir = 1;
	    if ( ($stage == 2) && (-e $Big_tmp.'/'.$filename) )
	    {
		$archive = $Big_tmp.'/'.$filename;
	    }
	    else
	    {
		$archive = @user_stats[7].'/'.$filename;
	    }
	    open $file, $archive;
	    print ERR "archive: $archive\n" if ($BUG);
	} else {
	    error(eval("\"".Locale::gettext::dgettext("ui", "3650")."\""));
	}
    } else {
	print ERR "file from web browser...\n" if ($BUG);
	my @fni=split( /\\/, $file );
	$filename = $fni[ $#fni ];
	print ERR "filename: $filename\n" if ($BUG);
	$homedir = 0;
	$archive = $Big_tmp.'/'.$filename;
	print ERR "archive: $archive\n" if ($BUG);
    }
}

sub do_restore
{
    my ($indx,$f,$key);

    close $file;

    if ( $bkmode eq 'user' || $bkmode eq 'site' ) {
	&validate_restore_filelist();
    }
    error(eval("\"".Locale::gettext::dgettext("ui", "3658")."\""))  unless (@sFiles);

    foreach $f (@sFiles) {
	next unless (exists $files{$f});
	$files{$f}{'error'} = eval("\"".Locale::gettext::dgettext("ui", "3651")."\"");
	if( $bkmode eq 'user' )
	{
	    unless ( $user eq $files{$f}{'user'} )
	    {
		$files{$f}{'error'} = eval("\"".Locale::gettext::dgettext("ui", "3652")."\"");
		next;
	    }
	    foreach $key (keys %userPaths)
	    {
		if ( ($userPaths{$key} eq '0') && ($f eq $userPaths{$key}) && ( ! -e $f))
		{
		    $files{$f}{'error'} = eval("\"".Locale::gettext::dgettext("ui", "3653")."\"");
		    next;
		}
	    }
	}    
	$indx .= " \"$f\"";
    }
    # tar stores \ as \\, and handles it different on extract
    $indx =~ s/\\/\\\\\\/g;

    $| = 1;
    my $eek = 0;
    print "content-type: text/html \nCache-Control: no-cache\n\n" unless ($HtmlHeader);
    print "<body bgcolor=\"#ffffff\">" unless ($HtmlHeader);
    $HtmlHeader = 1;
    print "<!--";

    open(ARKIV, $archive) || error(eval("\"".Locale::gettext::dgettext("ui", "3654")."\""));
    open(TAR, "| $tar xzvpP --same-owner $indx > $good 2> $bad" ) || error(eval("\"".Locale::gettext::dgettext("ui", "3655")."\""));
    while (<ARKIV>) {
	if (/^%%END_XML/)
	{
	    while (<ARKIV>)
	    {
		print TAR;
                if ($eek == 100) { print "."; $eek =0; } else { $eek++; }
	    }
	}
    }
    close ARKIV;
    close TAR;
    print "-->\n";
    $| = 0;
}


sub gen_restore_filelist
{
    print ERR "*** generating list of files to restore *** BEGIN\n" if ($BUG);
    my $selected = 0;
    my %noRepeats;
    my ($index,$f,$g);
    if ($filecount) {
	$index = $filecount + 2;
	while($index > -1) {
	    $f = $q->param("file.$index");
	    if ($f) {
		# if a dir is selected, select everything underneath it
		if( $files{"$f"}{'md5'} eq '0' ) {
		    print ERR "$f is a directory, selecting everthing under this directory\n" if ($BUG);
		    push (@sFiles, $f) unless ($noRepeats{$f});
		    foreach $g (keys %files) {
			if ($g =~ /\Q^$f\//) {
			    push (@sFiles, $g) unless ($noRepeats{$g});
			    print ERR "\t$g selected as a file in sub-dir\n" if ($BUG);
			    $noRepeats{$g} = 1;
			}
		    }
		}
		elsif ($f =~ /^$Mailspool\/(\S+)/) {
		    # deal with both the link and the actual file 
		    my $auxpath;

		    if ($bkmode eq 'site' or $bkmode eq 'user') {
			# enforce site-specific settings
			$auxpath = $mailpath . '/' . $1; 
		    } else {
			$auxpath = $Userdir . '/' . 
			    (user_list_site($1))[1] . $sitemailpath . '/' . $1;
		    }
					   
		    print ERR "$f selected -- autoselecting $auxpath\n" if ($BUG);
		    push @sFiles, $f unless $noRepeats{$f};
		    push @sFiles, $auxpath unless $noRepeats{$auxpath};
		    $noRepeats{$auxpath} = 1;
		    ++$selected;
		} else
		{
		    print ERR "$f selected\n" if ($BUG);
		    push (@sFiles, $f) unless ($noRepeats{$f});
		}
		$noRepeats{$f} = 1;
		$selected++;
	    }
	    $index--;
	}
    } else {
	foreach $f (keys %files) {
	    print ERR "$f selected\n" if ($BUG);
	    push (@sFiles, $f) unless ($noRepeats{$f});
	    $noRepeats{$f} = 1;
	    $selected++;
	}
    }
    print ERR "*** generating list of files to restore *** END\n" if ($BUG);
}


sub sitepathOK
{
    my ($site, $path, @dirs) = @_;
    my $key;

    # double dots are only okay in the following directories:
    foreach $key (@dirs) {
	return 1 if ($path eq $key);
    }
    return ($path =~ /\.\./) ? 0 : 1;
}

sub validate_restore_filelist
{
    my ($key,$f,$keyuid);
    my (@tFiles,%noR,@tymp,%Pathz);
    my $myuid = @user_stats[2];
    print ERR "\n**Validating restore filelist**\n\n" if ($BUG);

    if ($bkmode eq 'user') {
	%Pathz = %userPaths;
    } elsif ($bkmode eq 'site') {
	%Pathz = %sitePaths;
    } else {
	return;
    }

    foreach $key (keys %Pathz)
    {
	print ERR "bkmode equals $bkmode\n" if ($BUG);
	if ((-e $key) && ($bkmode ne 'site'))
	{
	    @tymp = stat($key);
	    $keyuid = @tymp[4];
	    foreach $f (@sFiles) 
	    {
		print ERR "mode 1 (key $key) testing file $f...\n" if ($BUG);
		next unless ($f =~ /^$key/);
		next if (exists $noR{$f});
		next if ( ($f eq $key) && ($files{$f}{'md5'} eq '0'));
		print ERR " my uid = $myuid\tkey uid = $keyuid\n" if ($BUG);
		if ($keyuid ne $myuid)
		{
		    $badFiles{$f} = eval("\"".Locale::gettext::dgettext("ui", "3656")."\"");
		    next;
		}
		print ERR "$f ok\n" if ($BUG);
		push @tFiles, $f;
		$noR{$f} = 1;
	    }
	}
	elsif ($Pathz{$key})
	{
	    foreach $f (@sFiles)
	    {
		print ERR "mode 2 (key $key) testing file $f...\n" if ($BUG);
		next unless ($f =~ /^$key/);
		next if (exists $noR{$f});
		next unless sitepathOK($site, $f, @doubledotok);
 		print ERR "$f ok\n" if ($BUG);
		push @tFiles, $f;
		$noR{$f} = 1;
	    }
	}
	else
	{
	    foreach $f (@sFiles)
	    {
		print ERR "final mode 3 (key $key) testing file $f...\n" if ($BUG);
		next unless ($f =~ /^$key/);
		next if (exists $noR{$f});
		print ERR "no permission to create $f\n" if ($BUG);
		$badFiles{$f} = eval("\"".Locale::gettext::dgettext("ui", "3657")."\"");
		$noR{$f} = 1;
	    }
	}
    }
    @sFiles = @tFiles;
}

sub do_report
    # checks to see if error output was displayed, then verifies that all files
    # have been restored and have valid md5 checksums
    # generates a report in HTML format
{
    my (@goodF,@goodD,@badF,$f,$md5,$temp,$key,@tymp);

#    print "content-type: text/html \nCache-Control: no-cache\n\n";
#    print "<head>";
#    print "<body bgcolor=\"#FFFFFF\">";
    print "<script language=javascript>";
    print "top.showInfo( '' )";
    print "</script>\n";
    print "</head>";
    foreach $f (@sFiles) {
	if ( -d $f ) { push @goodD, $f; next; }
	unless ( -e $f ) {
	    push @badF, $f;
	    next;
	}
	$md5 = `md5sum \"$f\"` if (-e $f);
	$md5 =~ /^(\S+)  .*$/;
	$md5 = $1;
	if ( $files{$f}{'md5'} eq $md5 ) {
	    push @goodF, $f;
	} else {
	    push @badF, $f;
	}
    }
    if (@goodD) {
	print "<h1>".eval("\"".Locale::gettext::dgettext("ui", "restore_report_dirs")."\"")."</h1><ul>\n";
	foreach $f (@goodD) {
	    printf( "<li><b>%s</b>\n", $f );
	}
	print "</ul><br>\n";
    }
    if (@goodF)
    {
	print "<h1>".eval("\"".Locale::gettext::dgettext("ui", "restore_report_valid")."\"")."</h1><ul>\n";
	foreach $f (@goodF)
	{
	    printf( "<li><b>%s</b> &nbsp; &nbsp; [%s bytes] &nbsp; &nbsp; [%s]\n", $f, $files{$f}{'size'}, $files{$f}{'mtime'} );
	}
	print "</ul><br>\n";
    }
    if (@badF)
    {
	print "<h1>".eval("\"".Locale::gettext::dgettext("ui", "restore_report_invalid")."\"")."</h1><ul>\n";
	foreach $f (@badF)
	{
	    printf( "<li><b>%s</b>&nbsp;&nbsp; %s\n", $f, $files{$f}{'error'} );
	}
	print "</ul><br>\n";
    }
    if (%badFiles)
    {
	print "<h1>".eval("\"".Locale::gettext::dgettext("ui", "restore_report_errors")."\"")."</h1><ul>\n";
	foreach $key (keys %badFiles)
	{
	    printf( "<li><b>%s</b>&nbsp&nbsp %s\n", $key, $badFiles{$key} );
	}
	print "</ul><br>\n";
    }
    print "<center><a href=\"restoremenu.cgi\"><img src=\"/.cobalt/images/back_but\" border=0></a></center>";
}

sub read_backupheader
# Process the entire backup header breaking it up into
# usable peices for the resore process
# Effects:
#   - Populates @index with file info
#   - Populates %header with info about how and when the backup
#     was performed
#   - Adds the entire header to $top
{
    # $fullname is the path and filename concatenated
    # @d contains each elemet of the fullnane
    #   (ie /path/to/file is $d[0]=path $d[1]=to $d[2]=file)
    # $depth is the index of @d
    # $valid makes sure %%<header> is intact
    # $backupfile
    my ($fullname,@d,$depth,$valid,$backupfile);
    $valid = 0;
    $backupfile = $file;

    # RED loop steps over the entire backupfile
    RED: while (<$file>) {
      if ($_ =~ /^%%BACKUP_HEADER/) {
	  $valid++;
	  $top .= $_;

	  # BLUE loop steps over the entire header
          BLUE: while (<$file>) {
	    $top .= $_;
	    last BLUE if (/^%%END_HEADER/);
	    next BLUE unless /^(\w+)=(.*)/;
	    $header{$1} = $2;
	  } # end BLUE
	  
	  next RED;
      } #endif
      
      if ($_ =~ /^%%BACKUP_INDEX/) {
	  $valid++;
	  $top .= $_;
	  
	  # GREEN loops process the entire backup index
	  GREEN: while (<$file>) {
	    $top .= $_;
	    last GREEN if (/^%%END_INDEX/);

	    # path filename mode user group size mtime dir=0/md5
	    # directories have no md5sum
	    next GREEN unless /^\'(.*?)\' \'(.*?)\' (\S+) (\S+) (\S+) (\S+) (\S+) (.*)\n/;
	    $fullname = $1.'/'.$2;
            @d=split(/\//, $1);
            $depth = $#d;

	    # An anon HOH?
	    %{ $files{$fullname} } = ( 'fname'   => $fullname,
				       'path'    => $1,
				       'name'    => $2,
				       'mode'    => $3,
				       'user'    => $4,
				       'group'   => $5,
				       'size'    => $6,
				       'depth'   => $depth,
				       'md5'     => $7,
				       'mtime'   => $8 );

	    push @index, "\'$1\' \'$2\' $3 $4 $5 $6 $7 $depth $8";
	    
	  } # end GREEN

	  last RED;
       } # endif
    } # end RED
    
    if ($valid < 2) { 
      &error(eval("\"".Locale::gettext::dgettext("ui", "3601")."\"")); 
    } 
}

sub write_backuparchive
{
	open ARCHIV, "> $archive";
	print ARCHIV $top;
	while (<$file>) {
		print ARCHIV;
	}
	close ARCHIV;
}

sub gen_filelist
{
    my ($key,$dir,$root,$m,$u);
    my (%roots,@dirs,@mails,@vacs,@dp);

    foreach $key (keys %files) {
        push @dirs, $key if ( $files{$key}{'md5'} eq '0');
	push @mails, $key if ( $files{$key}{'path'} eq $Mailspool);
	push @vacs, $key if ( $files{$key}{'path'} eq $Vaca_dir);
    }

    # determine who is doing restore
    # either mode=user and user=siteadmin
    #        mode=admin
    #        mode=user
    if ($bkmode eq 'user') {
	foreach $m (@mails)
	{
	    $filecount++;
	    if ($m ne "$Mailspool/$user")
	    {
		$filelist .= blank(1)." &nbsp; <img src=\"".$icon_generic_secure.'">&nbsp;'.eval("\"".Locale::gettext::dgettext("ui", "restore_sel_mailspool")."\"")." <b><u>".$files{$m}{'name'}."</u></b>&nbsp;&nbsp;[".$files{$m}{'size'}." bytes]&nbsp;&nbsp;[".$files{$m}{'mtime'}."]<br>\n";
	    }
	    else
	    {
		$filelist .= blank(1)."<input type=checkbox name=\"file.$filecount\" value=\"$m\"> &nbsp; <img src=\"".$icon_mail.'">&nbsp;'.eval("\"".Locale::gettext::dgettext("ui", "restore_sel_mailspool")."\"")." <b><u>".$files{$m}{'name'}."</u></b>&nbsp;&nbsp;[".$files{$m}{'size'}." bytes]&nbsp;&nbsp;[".$files{$m}{'mtime'}."]<br>\n";
	    }
	}	
	foreach $m (@vacs)
	{
	    $filecount++;
	    if ($m ne "$Vaca_dir/$user.msg")
	    {
		$filelist .= blank(1)." &nbsp; <img src=\"".$icon_generic_secure.'">&nbsp;'.eval("\"".Locale::gettext::dgettext("ui", "restore_sel_vacation")."\"")." <b><u>".$files{$m}{'name'}."</u></b>&nbsp;&nbsp;[".$files{$m}{'size'}." bytes]&nbsp;&nbsp;[".$files{$m}{'mtime'}."]<br>\n";
	    }
	    else
	    {
		$filelist .= blank(1)."<input type=checkbox name=\"file.$filecount\" value=\"$m\"> &nbsp; <img src=\"".$icon_mail.'">&nbsp;'.eval("\"".Locale::gettext::dgettext("ui", "restore_sel_vacation")."\"")." <b><u>".$files{$m}{'name'}."</u></b>&nbsp;&nbsp;[".$files{$m}{'size'}." bytes]&nbsp;&nbsp;[".$files{$m}{'mtime'}."]<br>\n";
	    }
	}	
    } elsif ($bkmode eq 'site') {
	my @siteusers = vsite_get_users($group);
	if ($siteusers[0] =~ /^[134]\S*\s/) {
	    @siteusers = ();
	}
	my ($foundmail);

        foreach $m (@mails) {
            $filecount++;
            $foundmail=0;
            foreach $u (@siteusers) {
                if ($m eq "$Mailspool/$u") {
		    $filelist .= blank(1)."<input type=checkbox name=\"file.$filecount\" value=\"$m\"> &nbsp; <img src=\"".$icon_mail.'">&nbsp;'.eval("\"".Locale::gettext::dgettext("ui", "restore_sel_mailspool")."\"")." <b><u>".$files{$m}{'name'}."</u></b>&nbsp;&nbsp;[".$files{$m}{'size'}." bytes]&nbsp;&nbsp;[".$files{$m}{'mtime'}."]<br>\n";
                    $foundmail=1;
                }
            }
            unless ($foundmail) {
		$filelist .= blank(1)." &nbsp; <img src=\"".$icon_generic_secure.'">&nbsp;'.eval("\"".Locale::gettext::dgettext("ui", "restore_sel_mailspool")."\"")." <b><u>".$files{$m}{'name'}."</u></b>&nbsp;&nbsp;[".$files{$m}{'size'}." bytes] &nbsp;&nbsp;[".$files{$m}{'mtime'}."]<br>\n";
            }
        }
         
        foreach $m (@vacs) {
            $filecount++;
            $foundmail=0;
            foreach $u (@siteusers) {
                if ($m eq "$Vaca_dir/$u.msg") {
		    $filelist .= blank(1)."<input type=checkbox name=\"file.$filecount\" value=\"$m\"> &nbsp; <img src=\"".$icon_mail.'">&nbsp;'.eval("\"".Locale::gettext::dgettext("ui", "restore_sel_vacation")."\"")." <b><u>".$files{$m}{'name'}."</u></b>&nbsp;&nbsp;[".$files{$m}{'size'}." bytes]&nbsp;&nbsp;[".$files{$m}{'mtime'}."]<br>\n";
                    $foundmail=1;
                }
            }    
            unless ($foundmail) {
		$filelist .= blank(1)." &nbsp; <img src=\"".$icon_generic_secure.'">&nbsp;'.eval("\"".Locale::gettext::dgettext("ui", "restore_sel_vacation")."\"")." <b><u>".$files{$m}{'name'}."</u></b>&nbsp;&nbsp;[".$files{$m}{'size'}." bytes]&nbsp;&nbsp;[".$files{$m}{'mtime'}."]<br>\n";
            }
        }
    } else {
	foreach $m (@mails) {
	    $filecount++;
	    $filelist .= blank(1)."<input type=checkbox name=\"file.$filecount\" value=\"$m\"> &nbsp; <img src=\"".$icon_mail.'">&nbsp;'.eval("\"".Locale::gettext::dgettext("ui", "restore_sel_mailspool")."\"")." <b><u>".$files{$m}{'name'}."</u></b>&nbsp;&nbsp;[".$files{$m}{'size'}." bytes]&nbsp;&nbsp;[".$files{$m}{'mtime'}."]<br>\n";
	}
	foreach $m (@vacs) {
	    $filecount++;
	    $filelist .= blank(1)."<input type=checkbox name=\"file.$filecount\" value=\"$m\"> &nbsp; <img src=\"".$icon_mail.'">&nbsp;'.eval("\"".Locale::gettext::dgettext("ui", "restore_sel_vacation")."\"")." <b><u>".$files{$m}{'name'}."</u></b>&nbsp;&nbsp;[".$files{$m}{'size'}." bytes]&nbsp;&nbsp;[".$files{$m}{'mtime'}."]<br>\n";
	}
    }

    # find unique top level dirs with lowest depths
    foreach $dir (@dirs) {
 	print ERR "DIR[1]: $dir\n" if ($BUG);
	while (parent($dir)) {
	    @dp = split( '/', $dir );
	    pop @dp;
	    $dir = join( '/', @dp );
	}
 	print ERR "DIR[2]: $dir\n" if ($BUG);
	$roots{$dir} = $files{$dir}{'depth'} unless (exists $roots{$dir});
    }
    foreach $root (keys %roots) {
  	print ERR "doing ROOT $root\n" if ($BUG);
	do_dir($root);
    }
}

sub do_dir
{
    my $root = @_[0];
    print ERR "doing DIR $root\n" if ($BUG);
    my (@dirs,@filez,$item);

    @dirs = get_dirs($root);
    @filez = get_files($root);
    print ERR "$root:  found subdirs @dirs\n" if ($BUG);
    print ERR "$root:  found files @filez\n" if ($BUG);

    $filecount++;

#    print ERR "backup mode: $bkmode\nsitehome: $sitehome\nroot: $root\n" if ($BUG);
    if (($bkmode eq 'user' && $root !~ "^$userhome") || ($bkmode eq 'site' && $root !~ "^$sitehome"))
    {
	$filelist .= blank($files{$root}{'depth'} + 1)." &nbsp; <img src=\"".$icon_folder_secure.'">&nbsp;<b>'.$root."</b><br>\n";
    }
    else
    {
	$filelist .= blank($files{$root}{'depth'})."<input type=checkbox name=\"file.$filecount\" value=\"$root\"> &nbsp; <img src=\"".$icon_folder.'">&nbsp;<b>'.$root."</b><br>\n";
    }

    foreach $item (@dirs) {
	do_dir( $item );
    }

    foreach $item (@filez) {
	$filecount++;
#	print ERR "backup mode: $bkmode\nsitehome: $sitehome\nroot: $root\n" if ($BUG);
	if (($bkmode eq 'user' && $root !~ "^$userhome") || ($bkmode eq 'site' && $root !~ "^$sitehome"))
	{
	    $filelist .= blank($files{$item}{'depth'} + 1)." &nbsp; <img src=\"".$icon_generic_secure.'">&nbsp;<b>'.$files{$item}{'name'}."</b>&nbsp;&nbsp;[".$files{$item}{'size'}." bytes]&nbsp;&nbsp;[".$files{$item}{'mtime'}."]<br>\n";
	}
	else
	{
	    $filelist .= blank($files{$item}{'depth'})."<input type=checkbox name=\"file.$filecount\" value=\"$item\"> &nbsp; <img src=\"".$icon_generic.'">&nbsp;<b>'.$files{$item}{'name'}."</b>&nbsp;&nbsp;[".$files{$item}{'size'}." bytes]&nbsp;&nbsp;[".$files{$item}{'mtime'}."]<br>\n";
	}
    }

    return;
} 

# takes a root directory, and returns all the files under this root,
# not traversing the tree at all
sub get_files
{
    my $key;
    my $root = @_[0];
    my @filez;
    foreach $key (keys %files) {
	push @filez, $key if ( ($files{$key}{'md5'} ne '0') && ($files{$key}{'path'} eq $root) );
    }
    return @filez;
}

# takes a root directory, and returns all the directories under this root,
# not traversing the tree at all
sub get_dirs
{
    my $key;
    my $root = @_[0];
    my @dirs;
    foreach $key (keys %files) {
	push @dirs, $key if ( ($files{$key}{'md5'} eq '0') && ($files{$key}{'path'} eq $root) );
    }
    return @dirs;
}

# returns the parent of the passed file/dir
sub parent
{
    my $f = @_[0];
    my $p = $files{$f}{'path'};
    print ERR "DIR: $f\tPARENT: $p\n" if ($BUG);
    return 1 if (exists $files{$p});
    print ERR "PARENT does not exist\n" if ($BUG);
    return 0;
}

sub blank
{
    my ($i,$b,$c);
    $b = "<img src=\"/cobalt-images/blank\">";
    for ($i = 0; $i < @_[0]; $i++) {
	$c .= $b;
    }
    return $c;
}

sub do_cleanup
{
    close ERR if ($BUG);
    unlink $bad if (-e $bad);
    unlink $good if (-e $good);
    if (!$file || $homedir == 0) { unlink $archive if (-e $archive); }
}

sub error
{
    my $UserError = '<IMG SRC="/.cobalt/images/bang" ALIGN=CENTER>'."@_";

    open (TEMPLATE, "$Template_dir/restoremenu.tmpl")
	|| die "Cannot read $Template_dir/restoremenu.tmpl: $!\n";

    print "content-type: text/html \nCache-Control: no-cache\n\n" unless ($HtmlHeader);
    while( <TEMPLATE> ) {
	# Do our substitutions
	s/\[USER\]/$user/o;
	s/\[BACKUPS\]//o;
	s/\[USERERROR\]/$UserError/o;
	s/\[MODE\]/$bkmode/o;
	s/\[GROUP\]/$group/o;

	s/\[str.([^\]]+)\]/Locale::gettext::dgettext("ui", "$1")/geo;

	# Write out the form...
	print;
    }
    close TEMPLATE;

    exit 0;
}

