#!/usr/bin/perl

# author: Kevin K.M. Chiu
#       Cobalt Microserver
# version: 4.0
# date: 7-6-99
#
# version 1.0 supports users and groups
# version 2.0 supports sites
# Version 3.0 uses Monitor.pm instead of AlertDB.pm
#
# Copyright 1999-2000, Cobalt Networks, Inc
# All rights reserved

### START HARDCODED PATHS
my $AdminUser = 'admin';
my $BytesPerBlock = '1024';
my $Htgroup_dir = '/home/sites';
my $Template_dir = '/usr/admserv/templates';
my $Vol_home = 'home';
my $Vol_rootdir = '/vol';
my %defcon_images = ('0','<IMG SRC="/.cobalt/images/transparentball" BORDER=0 WIDTH=20 HEIGHT=15>','1','<IMG SRC="/.cobalt/images/greenball" BORDER=0 WIDTH=20 HEIGHT=15>','2','<IMG SRC="/.cobalt/images/yellowball" BORDER=0 WIDTH=20 HEIGHT=15>','3','<IMG SRC="/.cobalt/images/redball" BORDER=0 WIDTH=20 HEIGHT=15>');

my $Debug = (-e "/etc/DEBUG") ? 1 : 0;
$^W = $Debug;
### END HARDCODED PATHS
use Cobalt::Group;
use Cobalt::List;
use Cobalt::Locale;
use Cobalt::Netutil;
use Cobalt::Quota;
use Cobalt::User;
use Cobalt::DiskUtils;
use Cobalt::Vsite;
use Cobalt::swatch;
use Cobalt::Util;

# use strict;
use CGI;
use Locale::gettext;


# global variables
my $q=new CGI;

my $alertScriptURL='/cgi-bin/.cobalt/alert/alert.cgi';
my $forbiddenURL='/.cobalt/error/forbidden.html';
my $diskUsageScriptURL='/cgi-bin/.cobalt/diskUsage/diskUsage.cgi';

my $locale=Cobalt::Locale->getByHTTPHeader( $ENV{ 'HTTP_ACCEPT_LANGUAGE' } );
if( $locale ) { $ENV{ 'LANGUAGE' }=$locale->getLocale(); }

my $alert=$q->param( 'alert' );

# get which page to show
my $show=$q->param( 'show' );
my $site=$q->param( 'site' );

my $user = $ENV{REMOTE_USER};
unless (user_authok($user, "admin") or
	(($show eq 'oneSite') and (user_authok($user, "group", $site)))) {
    print "location: $forbiddenURL\n\n";
    if ($show eq 'oneSite') {
	die "Unauthorized access by $user, not a member of $site.\n"; 
    } else {
        die "Unauthorized access by $user, not $AdminUser.\n";
    }
}

# check to see if everything is set up ok. if we aren't set up
# yet, just spit out a message.
my $defcon = swatch_get_defcon_disk();
unless ($defcon) {
    print "content-type: text/html\nCache-Control: no-cache\n\n";
    open( TEMPLATE, "$Template_dir/diskUsageNoReport.tmpl" )
	|| die "Cannot read $Template_dir/diskUsageNoReport.tmpl: $!\n";
    while (<TEMPLATE>) {
	s/\[SEVERITY_LOGO\]/$defcon_images{$defcon}/o;
	s/\[MESSAGE\]/MSG_get_ui("swatch_body_defcon_$defcon")/eo;
        s/\[ALERTSCRIPTURL\]/$alertScriptURL/o;
	s/\[str.([^\]]+)\]/MSG_get_ui("$1")/geo;
	print;
    }
    exit 0;
}

# find the list of host associated groups and site names
my @groups;
my @siteNames;
my @groupSeverityLogos;		# severity for an entire site
my @siteSeverityLogos;		# severity for site quota violation only
my %groupToNum;

# add virtual sites
my @vsiteInfo=vsite_list();
my $i;
for( $i=0; $i<=$#vsiteInfo; $i++ ) {
    my $groupName=$vsiteInfo[ $i ][ 2 ];

    push @siteNames, $vsiteInfo[ $i ][ 1 ];
    push @groups, $groupName;
    push @groupSeverityLogos, $defcon_images{ swatch_get_defcon_quota_member($groupName) };
    push @siteSeverityLogos, $defcon_images{ swatch_get_defcon_quota_vsite($groupName) };

    $groupToNum{ $groupName }=$i;
}

my %mounts = disk_getmounts();
my ($label,%totalFrees);
foreach (keys %mounts) {
    $label = disk_getlabel($mounts{$_}{MountPoint});
    if ($label) {
	$totalFrees{$label} = $mounts{$_}{Free} * $BytesPerBlock/1048576;
    }
}

my @groupUsages;
my @groupFrees;
my @groupQuotas;
my @groupUsagePercents;
my @groupFreePercents;
my @groupQuotaStrs;
my @groupUsageBars;
my @groupFreeBars;
my $allSitesUsage=0;
my $group;
my $site_home;
foreach $group ( @groups ) {
    my ( $groupUsage, $groupQuota )=( group_get_quota( $group ) );

    # block to MB
    $groupQuota*=$BytesPerBlock/1048576;
    $groupQuota=int( $groupQuota*10 )/10;
    $groupUsage*=$BytesPerBlock/1048576;
    $groupUsage=int( $groupUsage*10 )/10;

    $site_home = disk_getlabel("$Htgroup_dir/$group");
			  
    # Note: disk usage can be larger than quota because it is a soft quota
    my $groupFree;
    if( $groupQuota==0 ) { $groupFree=$totalFrees{$site_home}; }
    else {
	$groupFree=( $groupQuota>$groupUsage )?$groupQuota-$groupUsage:0;
	# quota can be larger than physical size
	if( $groupFree>$totalFrees{$site_home} ) { $groupFree=$totalFrees{$site_home}; }
    }
    $groupFree=int( $groupFree*10 )/10;

    push @groupUsages, $groupUsage;
    push @groupQuotas, $groupQuota;
    push @groupFrees, $groupFree;

    $allSitesUsage+=$groupUsage;

    # Note: disk usage can be larger than quota because it is a soft quota
    my $groupUsagePercent;
    my $groupFreePercent;

    $groupUsagePercent=int( 100*$groupUsage/( $groupUsage+$groupFree ) );
    $groupFreePercent=100-$groupUsagePercent;

    my $groupQuotaStr;
    if( $groupQuota==0 ) { $groupQuotaStr='<I>Unlimited</I>'; }
    else { $groupQuotaStr=$groupQuota; }

    my $groupUsageBar;
    if( $groupUsagePercent>0 ) { $groupUsageBar="<IMG SRC=\"/.cobalt/images/amber_color1\" WIDTH=$groupUsagePercent HEIGHT=10 BORDER=0>"; }
    my $groupFreeBar;
    if( $groupFree>0 ) { $groupFreeBar="<IMG SRC=\"/.cobalt/images/green_color\" WIDTH=$groupFreePercent HEIGHT=10 BORDER=0>"; }

    push @groupUsagePercents, $groupUsagePercent;
    push @groupFreePercents, $groupFreePercent;
    push @groupQuotaStrs, $groupQuotaStr;
    push @groupUsageBars, $groupUsageBar;
    push @groupFreeBars, $groupFreeBar;
}
# block to MB
$allSitesUsage=int( $allSitesUsage*10 )/10;


# get which page to show
if ( $show eq 'allSites' ) {
    genAllSitesPage();
} elsif ( $show eq 'oneSite' ) {
    my $back=$q->param( 'back' );
    genSitePage( $site, $back, 
		 $totalFrees{disk_getlabel("$Htgroup_dir/$site")});
} else {
    # show the summary page
    genSummaryPage();
}

1;

sub genSummaryPage {
    my $back='';
    if($alert) {
	$back = "<A HREF=\"$alertScriptURL\"><IMG SRC=\"/.cobalt/images/back_but\" BORDER=0></A>";
    }

    my $fmt = "%.1f";

    my $sysObj = swatch_get("quota", "sys");
    my $sysTotal = sprintf $fmt, $sysObj->get("total");
    my $sysFree = sprintf $fmt, $sysObj->get("free");
    my $sysUsed = sprintf $fmt, $sysObj->get("used");
    my $sysPercent = sprintf $fmt, $sysObj->get("percent");
    my $sysDefcon = swatch_get_defcon("quota", "sys");
    
    my $siteObj = swatch_get("quota", "all");
    my $siteTotal = sprintf $fmt, $siteObj->get("total") - $sysTotal;
    my $siteFree = sprintf $fmt, $siteObj->get("free") - $sysFree;
    my $siteUsed = sprintf $fmt, $siteObj->get("used") - $sysUsed;
    my $sitePercent = ($siteTotal) ? 
	sprintf $fmt, $siteUsed / $siteTotal * 100 : '?';

    # The values are given in bytes, divide by 1048576 to get megs
    my $siteLight = $defcon_images{ swatch_get_defcon_sites() };
    my $sysLight = $defcon_images{ $sysDefcon };

    my $mb = MSG_get_ui("megabyte");
    my $dhtml;
    my $homedisk = swatch_get("quota", "/$Vol_home");
    my $name = $Vol_home;
    my $light = $defcon_images{ swatch_get_defcon("quota", "/$Vol_home") };
    my $total = sprintf $fmt, $homedisk->get("total");
    my $free = sprintf $fmt, $homedisk->get("free");
    my $percent = sprintf $fmt, $homedisk->get("percent");
    
    my $dhtml = <<END;
<TR>
    <TD WIDTH="26">
	<CENTER><A HREF="javascript: top.showInfo( volUsageHelp );" onMouseOver="status='[str.help]'; top.showInfo( volUsageHelp ); return true;"><IMG SRC="/.cobalt/images/smallest_help_but" ALIGN="BOTTOM" BORDER="0"></A></CENTER>
    </TD>
    <TD WIDTH="26">
	<CENTER>$light</CENTER>
    </TD>
    <TD ALIGN="LEFT" WIDTH="260">
    <FONT COLOR="#FFFFFF">&nbsp;</FONT>
	<B>$name</B>
    </TD>
    <TD ALIGN="RIGHT" WIDTH="95">
	$total $mb
    </TD>
    <TD ALIGN="RIGHT" WIDTH="95">
	$free $mb
    </TD>
    <TD ALIGN="RIGHT" WIDTH="73">
	$percent
    </TD>
</TR>
END
    ;

    my @disks = Cobalt::Meta::query("type"  => "quota",
				    "where" => ["name", "~", "$Vol_rootdir"]);

    foreach (@disks) {
	$name = $_->getname;
	$light = $defcon_images{ swatch_get_defcon("quota", $name) };
	$name =~ s|/vol/(.*)|$1|;
	$total = sprintf $fmt, $_->get("total");
	$free = sprintf $fmt, $_->get("free");
	$percent = sprintf $fmt, $_->get("percent");

	$dhtml .= <<END;
<TR>
    <TD WIDTH="26">
	<CENTER><A HREF="javascript: top.showInfo( volUsageHelp );" onMouseOver="status='[str.help]'; top.showInfo( volUsageHelp ); return true;"><IMG SRC="/.cobalt/images/smallest_help_but" ALIGN="BOTTOM" BORDER="0"></A></CENTER>
    </TD>
    <TD WIDTH="26">
	<CENTER>$light</CENTER>
    </TD>
    <TD ALIGN="LEFT" WIDTH="260">
	<FONT COLOR="#FFFFFF">&nbsp;</FONT>
	<B>$name</B>
    </TD>
    <TD ALIGN="RIGHT" WIDTH="95">
	$total $mb
    </TD>
    <TD ALIGN="RIGHT" WIDTH="95">
	$free $mb
    </TD>
    <TD ALIGN="RIGHT" WIDTH="73">
	$percent
    </TD>
</TR>
END
    ;
    }


    # Read in the template
    open( TEMPLATE, "$Template_dir/diskUsageSum.tmpl" )
	|| die "Cannot read $Template_dir/diskUsageSum.tmpl: $!\n";

    print "content-type: text/html\nCache-Control: no-cache\n\n";

    while ( <TEMPLATE> ) {
	s/\[SYSLIGHT\]/$sysLight/o;
	s/\[SYSTOTAL\]/$sysTotal/o;
	s/\[SYSFREE\]/$sysFree/o;
	s/\[SYSPERCENT\]/$sysPercent/o;

	s/\[SITELIGHT\]/$siteLight/o;
	s/\[SITETOTAL\]/$siteTotal/o;
	s/\[SITEFREE\]/$siteFree/o;
	s/\[SITEPERCENT\]/$sitePercent/o;

	s/\[SITEUSAGEURL\]/$diskUsageScriptURL?show=allSites&alert=$alert/o;
	s/\[BACK\]/$back/o;

	s/\[DISKINFO\]/$dhtml/o;

	s/\[str.([^\]]+)\]/Locale::gettext::dgettext("ui", "$1")/geo;

	print;
    }

    close( TEMPLATE );
}


sub genAllSitesPage {
    my $usageRow;
    my $i;
    for( $i=0; $i<=$#groups; $i++ ) {
	$usageRow.=<<END;
<TR ALIGN="CENTER">
  <TD><P ALIGN="CENTER">$groupSeverityLogos[ $i ]</TD>
  <TD><P ALIGN="CENTER"><A HREF="$diskUsageScriptURL?show=oneSite&site=$groups[ $i ]&back=t&alert=$alert">$siteNames[ $i ]</A></TD>
  <TD><P ALIGN="CENTER">$groupUsages[ $i ]</TD>
  <TD><P ALIGN="CENTER">$groupFrees[ $i ]</TD>
  <TD><P ALIGN="CENTER">$groupQuotaStrs[ $i ]</TD>
  <TD NOWRAP><P ALIGN="LEFT">$groupUsageBars[ $i ]$groupFreeBars[ $i ]<FONT COLOR=#FFFFFF>&nbsp;</FONT>$groupUsagePercents[ $i ]%</TD>
</TR>
END
    }

    # Read in the template
    open( TEMPLATE, "$Template_dir/diskUsageSites.tmpl" )
        || die "Cannot read $Template_dir/diskUsageSites.tmpl: $!\n";

    print "content-type: text/html\nCache-Control: no-cache\n\n";

    while( <TEMPLATE> ) {
	s/\[USAGE_ROW\]/$usageRow/o;
        s/\[SUMUSAGEURL\]/$diskUsageScriptURL?alert=$alert/o;

	s/\[str.([^\]]+)\]/Locale::gettext::dgettext("ui", "$1")/geo;

        print;
    }

    close( TEMPLATE );
}


# argument 0 is the group name of the site
# argument 1 is true if a back button is needed
sub genSitePage {
    my ( $group, $back, $siteFree )=@_;

    my $groupNo=$groupToNum{ $group };

    my $siteName=$siteNames[ $groupNo ];

    my @users=vsite_get_users($group);
    if ($users[0] =~ /^[134]\S*\s/) {
	# substr( $users[0], 5 ).'<BR>';
	@users = ();
    }

    my $usageRow;
    my $user;
    foreach $user ( @users ) {
	my $usage;
	my $free;

	my $userObj = Cobalt::Meta->new("type" => "users");
	$userObj->retrieve($user);
	my $fullName = $userObj->get("fullname");
	my $userQuota = int($userObj->get("quota"));
	my $userUsage = (user_show($user))[3];

	# block to MB
	$userUsage*=$BytesPerBlock/1024/1024;
	$userUsage=int( $userUsage*10 )/10;

	# Note: disk usage can be larger than quota because it is a soft quota
	my $userFree;
	if( $userQuota==0 ) {      # 0 quota means unlimited
	    $userFree= $siteFree;

	    $usage=int( $userUsage/$total*100 );
	}
	else {
	    $userFree=( $userQuota>$userUsage )?$userQuota-$userUsage:0;
	    # quota can be larger than physical size
	    if( $userFree>$siteFree ) { $userFree=$siteFree; }

	    $usage=int( 100*$userUsage/( $userUsage+$userFree ) );
	}
	$userFree=int( $userFree*10 )/10;

	$free=100-$usage;

	my $severityLogo=$defcon_images{ swatch_get_defcon_quota_user($user) };

	my $quota;
	if( $userQuota==0 ) { $quota='<I>Unlimited</I>'; }
	else { $quota=$userQuota; }

	my $usageBar;
	if( $usage>0 ) { $usageBar="<IMG SRC=\"/.cobalt/images/amber_color1\" WIDTH=$usage HEIGHT=10 BORDER=0>"; }
	my $freeBar;
	if( $free>0 ) {  $freeBar="<IMG SRC=\"/.cobalt/images/green_color\" WIDTH=$free HEIGHT=10 BORDER=0>"; }

	$usageRow.=<<END;
<TR ALIGN="CENTER">
  <TD><P ALIGN="CENTER">$severityLogo</TD>
  <TD><P ALIGN="CENTER">$user</TD>
  <TD><P ALIGN="CENTER">$fullName</TD>
  <TD><P ALIGN="CENTER">$userUsage</TD>
  <TD><P ALIGN="CENTER">$userFree</TD>
  <TD><P ALIGN="CENTER">$quota</TD>
  <TD NOWRAP><P ALIGN="LEFT">$usageBar$freeBar<FONT COLOR=#FFFFFF>&nbsp;</FONT>$usage%</TD></TR>
END
    }

    if( $#users<0 ) {
	$usageRow="<TR><TD COLSPAN=7><FONT COLOR=#FFFFFF>&nbsp;</FONT><I>".eval("\"".Locale::gettext::dgettext("ui", "noUsers")."\"")."</I></TD></TR>";
    }

    my $hideBackStart='<!-- ';
    my $hideBackEnd=' -->';
    if( $back ) {
	$hideBackStart='';
	$hideBackEnd='';
    }

    print "content-type: text/html\nCache-Control: no-cache\n\n";

    # Read in the template
    open( TEMPLATE, "$Template_dir/diskUsageUsers.tmpl" )
	|| die "Cannot read $Template_dir/diskUsageUsers.tmpl: $!\n";

    while( <TEMPLATE> ) {
	s/\[HIDEBACKEND\]/$hideBackEnd/o;
	s/\[HIDEBACKSTART\]/$hideBackStart/o;
	s/\[SITEFREE\]/$groupFrees[ $groupNo ]/o;
	s/\[SITEFREEBAR\]/$groupFreeBars[ $groupNo ]/o;
	s/\[SITEQUOTA\]/$groupQuotaStrs[ $groupNo ]/o;
	s/\[SITENAME\]/$siteName/o;
	s/\[SITEUSAGEURL\]/$diskUsageScriptURL?show=allSites&alert=$alert/o;
	s/\[SITESEVERITYLOGO\]/$siteSeverityLogos[ $groupNo ]/o;
	s/\[SITEUSAGE\]/$groupUsages[ $groupNo ]/o;
	s/\[SITEUSAGEBAR\]/$groupUsageBars[ $groupNo ]/o;
	s/\[SITEUSAGEPERCENT\]/$groupUsagePercents[ $groupNo ]/o;
	s/\[USAGE_ROW\]/$usageRow/o;

	s/\[str.([^\]]+)\]/Locale::gettext::dgettext("ui", "$1")/geo;

	print;
    }

    close( TEMPLATE );
}
