#!/usr/bin/perl

# author: Kevin K.M. Chiu
# version: 2.0
# date: 07-01-99
#
# Copyright 1998-1999, Cobalt Networks, Inc
# All rights reserved

### START HARDCODED PATHS
my $AdminCGIs_dir = '/usr/admserv/cgi-bin/.cobalt';
my $Template_dir = '/usr/admserv/templates';
my %System_users = ('mail','1','httpd','1','root','1','gopher','1','squid','1','guest','1','anonymous','1','halt','1','ftp','1','nobody','1','operator','1','games','1','adm','1','daemon','1','bin','1','shutdown','1','lp','1','sync','1','news','1','uucp','1');

my $Debug = (-e "/etc/DEBUG") ? 1 : 0;
$^W = $Debug;
### END HARDCODED PATHS
use Cobalt::List;
use Cobalt::Vsite;
use Cobalt::Locale;
use Cobalt::Netutil;
use Cobalt::User;
use Cobalt::Util;

# use strict;
use CGI;
use Locale::gettext;

# global variables
my $q=new CGI;

my $forbiddenURL='/.cobalt/error/forbidden.html';

my $locale=Cobalt::Locale->getByHTTPHeader( $ENV{ 'HTTP_ACCEPT_LANGUAGE' } );
if( $locale ) { $ENV{ 'LANGUAGE' }=$locale->getLocale(); }

# the page that run this script
my $page=$q->param( 'page' );

my $group=$q->param( 'group' );
my $user = $ENV{REMOTE_USER};
unless (user_authok($user, "group", $group)) {
    print "location: $forbiddenURL\n\n";
    die "Unauthorised access by $user, not a member of $group.\n";
} 

my $mailListAddScriptURL="/cgi-bin/.cobalt/mailListAdd/mailListAdd.cgi";
my $mailListListScriptURL="/cgi-bin/.cobalt/mailListList/mailListList.cgi";
my $mailListListScriptPath="$AdminCGIs_dir/mailListList/mailListList.cgi";

if( $page eq 'maillistadd' ) { handleMailListAddPage(); }
else {
    print "content-type: text/html\nCache-Control: no-cache\n\n";
    genMailListAddPage( \*STDOUT, $group );
}

1;


#
# event handler for add mailing list page
#
sub handleMailListAddPage {
    my $err=0;
    my $info='<IMG SRC="/.cobalt/images/bang" ALIGN=CENTER>';

    # read form input
    my $group=$q->param( 'group' );
    my $listName=$q->param( 'listname' );
    my $recipientsArea=$q->param( 'recipients' );
    my @userRecipients=$q->param( 'userrecipients' );
    my $password=$q->param( 'password' );
    my $spam=$q->param( 'spam' );
    my $open=$q->param( 'open' );

    # drop the first elemet (blank)
    if ($userRecipients[0] eq '') {
	shift @userRecipients;
    }

    # get individual recipients
    my @recipients=split /[\s,]/m, $recipientsArea;

    # error check
    my $recipientErr=0;
    my $allRecipientsOK=1;
    my @resultRecipients;
    my $recipient;
    foreach $recipient ( @recipients ) {
	if( !$recipient ) { next; }

	if( !netutil_test_emailaddr( $recipient ) ) {
	    if( $allRecipientsOK ) { $info.=eval("\"".Locale::gettext::dgettext("ui", "recipientsNotValid")."\""); $allRecipientsOK=0; }
	    $info.=" $recipient";

	    $recipientErr=1;
	}
	else { push @resultRecipients, $recipient; }
    }
    if( !$allRecipientsOK ) { $info.='<BR>'; }
    @recipients=@resultRecipients;

    # check if list name is ok
    my @usersInGroup = vsite_get_users($group);
    foreach ( @usersInGroup ) {
	if ($_ eq $listName) {
	    $err = 1;
	}
    }
    my @aliases = Cobalt::Meta::query("type"  => "users",
		                      "keys"  => ["aliases"],
		                      "where" => ["vsite", "=", "$group"]);
    foreach ( @aliases ) {
	if ($_ eq $listName) {
	    $err = 1;
	}
    }
    if ($err) {
	$info.=eval("\"".Locale::gettext::dgettext("ui", "listNameInUseByUser")."\"");
    }
    elsif( $listName=~/[^-_0-9a-z]/) {
	$info.=eval("\"".Locale::gettext::dgettext("ui", "listNameNotValid")."\"");
	$err=1;
    }
    elsif( $System_users{ $listName } ) {
	$info.=eval("\"".Locale::gettext::dgettext("ui", "listNameInUseByUser")."\"");
	$err=1;
    }
    else {
	# everything is fine

	# add mailing list
	my $fqdn = (Cobalt::Vsite::vsite_get_bygroup($group))[1];
	my $ret=list_create($group, $fqdn, $listName,
			    map("$_\@${fqdn}", @userRecipients), @recipients);
	if( $ret!~/^2/o ) {
	    $info.=substr( $ret, 4 );
	    $err=1;
	}
	
	# set mailing list password
	$ret = list_config_password($group, $fqdn, $listName, $password);
	if( $ret!~/^2/o ) {
	    $info.=substr( $ret, 4 );
	    $err=1;
	}

	# set the open subscription option
	if ($open) {
	    $ret=list_config_open($group, $fqdn, $listName);
	} else {
	    $ret=list_config_closed($group, $fqdn, $listName);
	}
	if( $ret!~/^2/o ) {
	    $info.=substr( $ret, 4 );
	    $err=1;
	}

	# set the restricted posting option
	if ($spam) {
	    $ret=list_config_norestrict($group, $fqdn, $listName);
	} else {
	    $ret=list_config_restrict($group, $fqdn, $listName);
	}
	if( $ret!~/^2/o ) {
	    $info.=substr( $ret, 4 );
	    $err=1;
	}
    }

    if( $err || $recipientErr ) {
	$ENV{ PARAM_GROUP }=$group;
	$ENV{ PARAM_INFO }=$info;
	require $mailListListScriptPath;
	delete $ENV{ PARAM_GROUP };
	delete $ENV{ PARAM_INFO };
    }
    else {
	$ENV{ PARAM_GROUP }=$group;
	require $mailListListScriptPath;
	delete $ENV{ PARAM_GROUP };
    }
}

#
# argument 0 is the file handle the page goes to
# argument 1 is the group
# argument 2 is the information shown to user
#
sub genMailListAddPage {
    my ( $handle, $group, $info )=@_;
    my $fqdn = (Cobalt::Vsite::vsite_get_bygroup($group))[1];

    # display users
    my $userList;
    my $selectlist = MSG_get_ui("mailListSelectList");

    # do not show admin
    my @users=vsite_get_users($group);
    if ($users[0] =~ /^[134]\S*\s/) {
	@users = ();
    }
    my($i,$list);
    @users = (sort @users);
    if (scalar(@users)) {
	foreach (@users) {
	    $list .= qq[ <OPTION VALUE="$_">$_\@$fqdn</OPTION>\n ];
	}
	$userList .= <<END;
<SELECT MULTIPLE NAME="userrecipients" SIZE="6">
<OPTION VALUE="">--- $selectlist ---</OPTION>
$list
</SELECT>
<CENTER>
<A HREF="javascript: setAllUsers();" onMouseOver="status='[str.selectAllUsers]'; return true;">
  <IMG SRC="/.cobalt/images/select_all_but" BORDER="0">
</A></CENTER>
END

    } else {
	my $noUsers=eval("\"".Locale::gettext::dgettext("ui", "noUsers")."\"");
	$userList = qq[ <FONT COLOR="#FFFFFF">&nbsp;</FONT><I>$noUsers</I>\n ];
    }

    my $cancelURL = $mailListListScriptURL . "?group=$group";
    my $mailListOpenHelp=MSG_get_ui("mailListOpenHelp", 'majordomo@'.$fqdn);

    # Read in the template
    open (TEMPLATE, "$Template_dir/mailListAdd.tmpl")
	|| die "Cannot read $Template_dir/mailListAdd.tmpl: $!\n";

    while( <TEMPLATE> ) {
	s/\[MAILLISTADDSCRIPTURL\]/$mailListAddScriptURL/o;
	s/\[MAILLISTLISTSCRIPTURL\]/$cancelURL/o;
	s/\[MAILLISTOPENHELP\]/$mailListOpenHelp/o;
	s/\[USER_INFO\]/$info/o;
	s/\[USERLIST\]/$userList/o;
	s/\[GROUP\]/$group/;

	s/\[str.([^\]]+)\]/Locale::gettext::dgettext("ui", "$1")/geo;

	print $handle $_;
    }
    close TEMPLATE;	    
}
