#!/usr/bin/perl

# Failover Management - Cluster Reporting
# Will DeHaan - Cobalt Networks, Inc.
# Copyright 2000

### START HARDCODED PATHS
my $Template_dir = '/usr/admserv/templates';
my %defcon_images = ('0','<IMG SRC="/.cobalt/images/transparentball" BORDER=0 WIDTH=20 HEIGHT=15>','1','<IMG SRC="/.cobalt/images/greenball" BORDER=0 WIDTH=20 HEIGHT=15>','2','<IMG SRC="/.cobalt/images/yellowball" BORDER=0 WIDTH=20 HEIGHT=15>','3','<IMG SRC="/.cobalt/images/redball" BORDER=0 WIDTH=20 HEIGHT=15>');

my $Debug = (-e "/etc/DEBUG") ? 1 : 0;
$^W = $Debug;
### END HARDCODED PATHS
use Cobalt::RAID; # Cobalt RAID monitoring serivces
use Cobalt::Locale;
use Locale::gettext;

my($Locale) = Cobalt::Locale->new();

###########
# entre main
#

# Parse CGI post
my(%fields) = &parseinfo();
my($USER_INFO) = $fields{'USER_INFO'};

# Global
my $iam = '/cgi-bin/.cobalt/raidUsage/raidUsage.cgi';

# http(s) invocation preferred
# die "Web administrative interface use only\n\n" unless($ENV{'REMOTE_ADDR'});

# parse raid state
# State #       State Description                       UI Descriptor
#  0            Unconfigured/disabled/na                gray
#  1            Enabled, working great (UU)             green
#  2            Sync (RAID rebuild), non-fatal          yellow
#  3            Fatal error, single disk operation      red

my(@RAID_state) = raid_get_state();

$RAID_state[0] ||= 3; # this CGI should never be invoked unless we're actively using RAID 

my($template_state, $complete_pixel_width, $remainder_pixel_width, $sync_complete, $sync_remainder, $template_states);

if ($RAID_state[0] == 2) # RAID Rebuild/resync
  {
  # Prep progress bar
  $sync_complete = $RAID_state[1]; # Sync in progress, numeric 0-100
  $sync_complete = 100 if ($sync_complete > 100); 
  $sync_complete = 0 unless ($sync_complete =~ /^[\.\d]+$/);
  $sync_remainder = 100 - $sync_complete;

  # Scale completion values from percent to double-proof
  $complete_pixel_width = $sync_complete * 4;
  $complete_pixel_width ||= 1; # some browsers can't render a 0 pixel image
  $remainder_pixel_width = $sync_remainder * 4;
  $remainder_pixel_width ||= 1;

  # ETA is rounded by RAID.pm
  $RAID_state[2] ||= '0.5';
  $RAID_state[2] = '0.5' if (($sync_complete == 100) && ($RAID_state[2] > 19));
  }

# Set refresh based on problem set
my($status_refresh) = 900; 
$status_refresh = 15 if ($RAID_state[0] == 2); 
$status_refresh = 45 if ($RAID_state[0] == 3);

$sync_complete = int($sync_complete + 0.5);

open( TMPL, "$Template_dir/raidUsage.tmpl") ||
  die "Content-type: text/html\n\nMissing file: $Template_dir/raidUsage.tmpl\n\n"; 
while(<TMPL>)
  {
  $template_states = $1 if (/\<HA STATE (\d[\s\d]*)\>/);
  if(/\<HA STATE [^\>]+\>/ ... /\<\/HA STATE\>/)
    {
    next unless ($template_states =~ /$RAID_state[0]/);
    }

  next if (/HA STATE/); # Entire lines ignored!

  if(/\[/o)
    {
    s/\[DEFCON(\d)\]/$defcon_images{ $1 }/o;

    s/\[IMGCOMPLETE\]/$complete_pixel_width/og;
    s/\[REMAINDER\]/$remainder_pixel_width/og;

    s/\[JUSTIFICATION\]/$RAID_state[1]/o;
    s/\[REFRESH\]/$status_refresh/o;
    s/\[IAM\]/$iam/o;

    s/\[USER_INFO\]//o; # error message
      
    s/\[str\.([^\]]+)\]/Locale::gettext::dgettext('ui', "$1")/geo;
    s/\[SYNC_COMPLETE\]/$sync_complete/;
    s/\[RAID_STATE\]/$RAID_state[2]/;
    }
              
  print;
  }
close( TMPL );

#
# Fin main
###########

#########################################################################
# Parse the CGI form and return as a hash.                              #
#########################################################################
#
# This is a pretty standard CGI parsing
sub parseinfo
    {
    my ($k,@equality,%fields);
    my $input = '';

    read(STDIN,$input,$ENV{"CONTENT_LENGTH"}) if ($ENV{"CONTENT_LENGTH"} > 0 );
    $input .= $ENV{"QUERY_STRING"}."&";

    foreach $_ ( split(/&/,$input) )
        {
        /(.*)=(.*)/;
        my $left = $1;
        my $right = $2;

        $left =~ s/\%(.{2})/sprintf("%c",hex($1))/eg;
        $left =~ tr/+/ /s;

        $right =~ s/\%(.{2})/sprintf("%c",hex($1))/eg;
        $right =~ tr/+/ /s;
        $right =~ s/[#\$\*\\\^\(\)\n]/ /og;
        $right =~ s/\+/ /g;
        $right =~ s/  / /g;
        $right =~ s/\"/\"\"/g;

        if ($fields{"$left"})
            {
            $fields{"$left"} .= ", $right" unless ($right eq "");
            }
        else
            {
            $fields{"$left"} = $right;
            }
        }

    $fields{"remote_addr"} = $ENV{"REMOTE_ADDR"};
    $fields{"remote_host"} = $ENV{"REMOTE_HOST"};

    return(%fields);
    }

