#!/usr/bin/perl

# Site FTP daemon control and configuration
#
# Copyright 1999 Cobalt Networks, Inc
# Will DeHaan <will@cobaltnet.com>
# Version 1.0

# If the authenticating user is the RaQ administrator named $Adminuser,
# then first post a configurable form and process its submission.
# If the user is a site admin, only report the existing configuration

### START HARDCODED PATHS
my $AdminCGIs_dir = '/usr/admserv/cgi-bin/.cobalt';
my $Adminuser = 'admin';
my $BytesPerBlock = '1024';
my $Template_dir = '/usr/admserv/templates';

my $Debug = (-e "/etc/DEBUG") ? 1 : 0;
$^W = $Debug;
### END HARDCODED PATHS
use Cobalt::Locale;
use Cobalt::Ftp;	# Anonymous config, quota control
use Cobalt::Vsite;	# Find IP used by site
use Cobalt::Group;  	# Site quota reporting
use Cobalt::Meta;
use Cobalt::User;
use Cobalt::Util;

# use strict;
use DB_File; 		# UI database reporting
use Fcntl qw(O_RDWR O_CREAT O_RDONLY);
use CGI;
use Locale::gettext;

## Globals
my($siteListScriptPath)="$AdminCGIs_dir/siteList/siteList.cgi";
my($showTemplate)="$Template_dir/showFTP.tmpl";
my($editTemplate)="$Template_dir/editFTP.tmpl";
my($siteFTP_script)="$AdminCGIs_dir/siteFTP/index.cgi";
my $forbiddenURL="/.cobalt/error/forbidden.html";

my ($group,$q,$page);
if ($ENV{'PARAM_GROUP'}) {
    $group = $ENV{'PARAM_GROUP'};
} else {
    $q=new CGI;
    $group = $q->param('group');
    $page=$q->param('page');
    my $user = $ENV{REMOTE_USER};
    unless (user_authok($user, "group", $group)) {
	print "location: $forbiddenURL\n\n";
	die "Unauthorised access by $user, not a member of $group.\n";
    } 
}

my $siteObj = Cobalt::Meta->create("type" => "vsite");
$siteObj->retrieve($group);

my ($anon,$maxUsers,$quota,$ftpquota,$ftpusers);
$anon = $siteObj->get("ftp");
$maxUsers = $siteObj->get("ftpusers");
$quota = $siteObj->get("ftpquota");
my $ipaddr = $siteObj->get("ipaddr");
$ftpquota = $quota;
$ftpusers = $maxUsers;

$quota*=$BytesPerBlock/1048576; # convert from block to megabyte

my $locale=Cobalt::Locale->getByHTTPHeader( $ENV{ 'HTTP_ACCEPT_LANGUAGE' } );
if( $locale ) { $ENV{ 'LANGUAGE' }=$locale->getLocale(); }

## main()
die "Invalid site affiliated group: $group\n" unless(group_exist($group));

my $ret;
if ($page && ($ENV{'REMOTE_USER'} eq $Adminuser)) {
    $ret=update_config();
    $anon = $siteObj->get("ftp");
    $maxUsers = $siteObj->get("ftpusers");
    $ftpquota = $siteObj->get("ftpquota");
    generate_config($ret);
} elsif ($ENV{'REMOTE_USER'} eq $Adminuser) {
    generate_config();	# Create the form
} else {
    report_config(); 	# Un-editable page, not a form
}
## fin main()

1;

## Subs
sub update_config
{
    my $anonftp;
    if ($q->param('ftp') eq 'on') {
	$anon=1;
	$anonftp = 'on';
    } else {
	$anon=0;
	$anonftp = 'off';
    }

    $quota=$q->param('ftpquota');
    $maxUsers=$q->param('ftpusers');
    
    my $ftpquota = $quota;
    $quota*=1048576/$BytesPerBlock; # megabyte to blocks
    
    return MSG_get_ui("ftp_invalid_quota") unless ($quota =~ /^\d+$/);
    return MSG_get_ui("ftp_invalid_users") unless ($maxUsers =~ /^\d+$/ && $maxUsers > 0);
    return MSG_get_ui("ftp_cant_overquota") if ($quota > (group_get_quota($group))[1]);
    
    # Configure server
    my($status)=ftp_config($ipaddr,1,$anon,$group,$maxUsers,$quota);
    return substr($status,4) if ($status !~ /^2/); # Check for errors

    $siteObj->put("ftp" => $anonftp,
		  "ftpusers" => $maxUsers,
		  "ftpquota" => $ftpquota);
    $siteObj->save;
	
    return;
}

sub generate_config
{ 
    my($info)=@_;
    my $err='<IMG SRC="/.cobalt/images/bang" ALIGN=CENTER>&nbsp;'.$info if ($info); 

    my $ftpChecked;
    if ($anon eq 'on') {
	$ftpChecked = 'CHECKED';
    }
    
    # display page
    print "content-type: text/html\nCache-Control: no-cache\n\n";
    open(TMPL,"$editTemplate") || die "Can not open template $editTemplate for reading: $!";
    while(<TMPL>)
    {
	#s/\[\]//g;
	s/\[USER_INFO\]/$err/g;
	s/\[MAXFTPUSERS\]/$maxUsers/g;
	s/\[FTPCHECKED\]/$ftpChecked/g;
	s/\[FTPQUOTA\]/$ftpquota/g;
	s/\[GROUP\]/$group/g;
	
	s/\[str.([^\]]+)\]/Locale::gettext::dgettext("ui", "$1")/geo;
	
	print;
    }
    close(TMPL);
} 

sub report_config
{
    my $rowSpan = 1;
    my $check = MSG_get_ui("disabled");
    my $configRow;

    if($anon eq 'on') {
	my $BCC_010=MSG_get_ui("BCC_010");
	my $BCC_011=MSG_get_ui("BCC_011");

	$check=<<END;
<FONT COLOR="#FFFFFF">&nbsp;&nbsp;</FONT> $ftpquota <FONT>&nbsp;</FONT>$BCC_010
<BR><FONT COLOR="#FFFFFF">&nbsp;&nbsp;</FONT> $maxUsers <FONT>&nbsp;</FONT>$BCC_011
END
    ;
  }

    # display page
    print "content-type: text/html\nCache-Control: no-cache\n\n";
    open(TMPL,"$showTemplate") || die "Can not open template $showTemplate for reading: $!";
    while(<TMPL>)
    {
	s/\[FTP\]/$check/g;
	s/\[ROWSPAN\]/$rowSpan/g;
	s/\[FTPCONFIG\]/$configRow/g;
	
	s/\[str.([^\]]+)\]/Locale::gettext::dgettext("ui", "$1")/geo;
	
	print;
    }
    close(TMPL);
}
