#!/usr/bin/perl

# author: Kevin K.M. Chiu
#       Cobalt Networks
# version: 2.0
# date: 7-6-99
#
# Copyright 1999-2000, Cobalt Networks, Inc
# All rights reserved

### START HARDCODED PATHS
my $AdminCGIs_dir = '/usr/admserv/cgi-bin/.cobalt';
my $Template_dir = '/usr/admserv/templates';

my $Debug = (-e "/etc/DEBUG") ? 1 : 0;
$^W = $Debug;
### END HARDCODED PATHS
use Cobalt::Admserv;
use Cobalt::Email;
use Cobalt::List;
use Cobalt::Locale;
use Cobalt::Netutil;
use Cobalt::Util;
use Cobalt::User;
use Cobalt::Vacation;
use Cobalt::Vsite;
use Cobalt::Meta;

# use strict;
use CGI;
use Locale::gettext;

# global variables
my $q=new CGI;

my $page=$q->param( 'page' );

my $group;
if( $ENV{ 'PARAM_GROUP' } ) { $group=$ENV{ 'PARAM_GROUP' }; }
else { $group=$q->param( 'group' ); }

my $siteUserEmailScriptURL='/cgi-bin/.cobalt/siteUserEmail/siteUserEmail.cgi';
my $siteUserListScriptURL='/cgi-bin/.cobalt/siteUserList/siteUserList.cgi';
my $siteUserListScriptPath="$AdminCGIs_dir/siteUserList/siteUserList.cgi";
my $forbiddenURL="/.cobalt/error/forbidden.html";

my $user = $ENV{REMOTE_USER};
unless (user_authok($user, "group", $group)) {
    print "location: $forbiddenURL\n\n";    
    die "Unauthorised access by $user, not a member of $group.\n";
}

my $locale=Cobalt::Locale->getByHTTPHeader( $ENV{ 'HTTP_ACCEPT_LANGUAGE' } );
if( $locale ) { $ENV{ 'LANGUAGE' }=$locale->getLocale(); }

if( $page eq 'siteuseremail' ) { handleSiteUserEmailPage(); }
else { genSiteUserEmailPage(); }

1;


#
# event handler for email page
#
sub handleSiteUserEmailPage {
    my $siteObj = Cobalt::Meta->create("type" => "vsite");
    $siteObj->retrieve($group);
    my $fqdn = $siteObj->get("fqdn");
    my $domain = $siteObj->get("domain");
    
    # read form input
    my $userName=$q->param( 'name' );
    my $forwardAddr=$q->param( 'forwardaddr' );
    my $aliasArea=$q->param( 'aliases' );
    my $vacationMode=$q->param( 'vacationmode' );
    my $vacationMsg=$q->param( 'vacationmsg' );
    
    # Get user from the database.
    my $userObj = Cobalt::Meta->create("type" => "users");
    $userObj->retrieve($userName);
    
    # get individual aliases
    my $ret;
    my $err=0;
    my $info='<IMG SRC="/.cobalt/images/bang" ALIGN=CENTER>';
    
    my (@resultAliases,$alias,@badalii);
    if ($aliasArea) {
	# get individual aliases
	my @aliases=split /[\s,]+/m, $aliasArea;
	my %aliasHash;
	map { map { $aliasHash{$_} = 1; }
	      split(' ', $_); } Cobalt::Meta::query("type"  => "users",
						    "keys"  => ["aliases"],
						    "where" => ["vsite", "=", "$group",
								"and", "name", "<>", "$userName"]);
	foreach $alias (@aliases) {
	    next unless ($alias);
	    if ($alias =~ /^\@(.+)/) {
		if ($1 !~ /\Q$fqdn\E$/ || exists($aliasHash{$alias})) {
		    push @badalii, $alias;
		    $err=1;
		} else {
		    push @resultAliases, $alias;
		}
	    } else {
		$alias =~ s/^([^@]+)\@.*/$1/;
		if (exists($aliasHash{$alias}) || $alias !~ /^[a-zA-Z0-9\@\.\-\_]+$/o) {
		    push @badalii, $alias;
		    $err=1;
		} else {
		    push @resultAliases, $alias;
		}
	    }
	}
    }
    if ($err) {
	$info.=MSG_get_ui("emailAliasNotValid");
	$info.=join(', ', @badalii);
    }
    if( $forwardAddr && !netutil_test_emailaddr( $forwardAddr ) ) {
	$info.=eval("\"".Locale::gettext::dgettext("ui", "forwardAddrNotValid")."\"");
	$err=1;
    }
    if (! $err) {
	# everything is fine

	# set forward to
	if( $forwardAddr ) {
	    # build list
            # $forwardAddr could actually be a comma separated list of aliases
            $ret = alias_set( $userName, $forwardAddr );
	    $userObj->put('forward', $forwardAddr);
	} else {
	    # delete list
	    $ret='2';
            if( alias_exist( $userName ) ) { $ret=alias_delete( $userName ); }
	    $userObj->put('forward', '');
	}
        if( $ret!~/^2/o ) {
            $info.=substr( $ret, 4 ).'<BR>';
            $err=1;
        }

	# set mail aliases
	my $a = join(' ', @resultAliases);
	$userObj->put("aliases" => $a);

	foreach $_ (@resultAliases) {
	    push @alii, (/\@/) ? "$_" : "$_\@$fqdn";
	}
	# add an alias to self
	push @alii, "$userName\@$fqdn";

	$ret=mail_virtuser_set_byuser( $userName, @alii );
	if( $ret!~/^2/o ) {
	    $info.=substr( $ret, 4 ).'<BR>';
	    $err=1;
	}

	# set vacation mode
	$ret=vacation_set_on( $userName, $vacationMode, $fqdn );
	$userObj->put("vacation", ( ( $vacationMode ) ? 'on' : 'off' ) );
	if( $ret!~/^2/o ) {
	    $info.=substr( $ret, 4 ).'<BR>';
	    $err=1;
	}
	# set vacation message
	$ret=vacation_set_message( $userName, $vacationMsg );
	if( $ret!~/^2/o ) {
	    $info.=substr( $ret, 4 ).'<BR>';
	    $err=1;
	}
    }

    if( $err ) { genSiteUserEmailPage( $info ); }
    else {
	$userObj->save();
	
	$ENV{ PARAM_GROUP }=$group;
	require $siteUserListScriptPath;
	delete $ENV{ PARAM_GROUP };
    }
}

#
# argument 0 is the infomation shown to user
#
sub genSiteUserEmailPage {
    my ( $info )=@_;

    my $siteObj = Cobalt::Meta->create("type" => "vsite");
    $siteObj->retrieve($group);
    my $fqdn = $siteObj->get("fqdn");

    my $userName = $q->param('name');
    my $userObj = Cobalt::Meta->create("type" => "users");
    $userObj->retrieve($userName);
    my $fullName = $userObj->get("fullname");

    my ($forwardMode, $forwardAddr, @forwardList);
    @forwardList = Cobalt::List::alias_get_vacationless( $userName );
    if ($forwardList[0] =~ /^[134]\S*\s/) {
	# substr( $forwardList[0], 5 ).'<BR>';
	@forwardList = ();
    }
    if (@forwardList) {
	$forwardMode = 'CHECKED';
	$forwardAddr=join ',', @forwardList;
    } else {
	$forwardMode = '';
    }

    my $aliases = $userObj->get("aliases");
    $aliases =~ s/\s+/\n/g;
    my $vacationMode=($userObj->get("vacation") eq 'on' ) ?'CHECKED':'';
    my $vacationMsg=Cobalt::Vacation::vacation_get_message($userName);

    print "content-type: text/html\nCache-Control: no-cache\n\n";

    # Read in the template
    open( TEMPLATE, "$Template_dir/siteUserEmail.tmpl" )
        || die "Cannot read $Template_dir/siteUserEmail.tmpl: $!\n";

    while( <TEMPLATE> ) {
	s/\[ALIASES\]/$aliases/o;
	s/\[FORWARDADDR\]/$forwardAddr/o;
	s/\[FULLNAME\]/$fullName/o;
	s/\[GROUP\]/$group/o;
	s/\[SITEUSEREMAILSCRIPTURL\]/$siteUserEmailScriptURL/o;
	s/\[SITEUSERLISTSCRIPTURL\]/$siteUserListScriptURL/o;
	s/\[USER_INFO\]/$info/o;
	s/\[USERNAME\]/$userName/o;
	s/\[VACATIONMODE\]/$vacationMode/o;
	s/\[VACATIONMESSAGE\]/$vacationMsg/o;

	s/\[str.([^\]]+)\]/Locale::gettext::dgettext("ui", "$1")/geo;

	print;
    }
    close TEMPLATE;
}
